# People Movement Behavior(PMB)
# ver:0.0.3
# 2019/02/21
#
# ver:0.0.4
# 2019/12/06
# ver:0.0.5
# 2020/04/21


import serial
import time
import struct
#import numpy as np

class header:
	version = 0
	platform = 0
	timestamp = 0
	totalPackLen = 0
	frameNumber = 0
	subframeNumber = 0
	chirpMargin = 0
	frameMargin = 0
	uartSendTime = 0
	trackProcessTime = 0
	numTLVs = 0
	checksum = 0
	

class PeopleMB:
	
	magicWord =  [b'\x02',b'\x01',b'\x04',b'\x03',b'\x06',b'\x05',b'\x08',b'\x07',b'\0x99']
	port = ""
	hdr = header
	
	# add for PMB interal use
	tlvLength = 0
	numOfPoints = 0
	# for debug use 
	dbg = False #Packet unpacket Check: True show message 
	sm = False #Observed StateMachine: True Show message
	plen = 16  #number bytes of per package type6:16 type7:68 type8:1
	
	def __init__(self,port):
		self.port = port
		print("***People Moving Behavior init***")
		
	def useDebug(self,ft):
		self.dbg = ft
		
	def stateMachine(self,ft):
		self.sm = ft
		
	def getHeader(self):
		return self.hdr
		
	def headerShow(self):
		print("***header***********") 
		print("Version:     \t%x "%(self.hdr.version))
		print("Platform:    \t%X "%(self.hdr.platform))
		print("Time Stamp:  \t%s "%(self.hdr.timestamp))
		print("TotalPackLen:\t%d "%(self.hdr.totalPackLen))
		print("PID(frame#): \t%d "%(self.hdr.frameNumber))
		print("subframe#  : \t%d "%(self.hdr.subframeNumber))
		print("Inter-frame Processing Time:\t{:d} us".format(self.hdr.trackProcessTime))
		print("UART Send Time:\t{:d} us".format(self.hdr.uartSendTime))
		print("Inter-chirp Processing Margin:\t{:d} us".format(self.hdr.chirpMargin))
		print("Inter-frame Processing Margin:\t{:d} us".format(self.hdr.frameMargin))
		print("numTLVs:     \t%d "%(self.hdr.numTLVs))
		print("Check Sum   :\t{:x}".format(self.hdr.checksum))
		
		
	#for class internal use
	def tlvTypeInfo(self,dtype,count,dShow):
		sbyte = 16
		pString = ""
		nString = "numOfPoints :"
		stateString = "V6"
		if dtype == 6:
			sbyte = 16
			pString = "Point Cloud TLV"
		elif dtype == 7:
			sbyte = 68
			pString = "Target Object TLV"
			nString = "numOfObjects:"
			stateString = "V7"
		elif dtype == 8:
			pString = "Target Index TLV"
			sbyte = 1
			stateString = "V8"
		else:
			sbyte = 1
			pString = "*** Type Error ***"
			stateString = 'idle'
		 
		retCnt = count - 8
		nPoint = retCnt / sbyte
		if dShow == True:
			print("-----[{:}] ----".format(pString))
			print("tlv Type({:2d}B):  \t{:d}".format(sbyte,dtype))
			print("tlv length:      \t{:d}".format(count)) 
			print("{:}      \t{:d}".format(nString,int(nPoint)))
			print("value length:    \t{:d}".format(retCnt))  
		
		return stateString, sbyte, retCnt, nPoint
		
#
# TLV: Type-Length-Value
# read TLV data
# input:
#     disp: True:print message
#			False: hide printing message
# output:(return parameter)
# (pass_fail, v6, v7, v8)
#  pass_fail: True: Data available    False: Data not available
#  v6: point cloud 2d infomation
#  v7: Target Object information
#  v8: Target Index information
#
	def tlvRead(self,disp):
		#print("---tlvRead---")
		#ds = dos
		typeList = [6,7,8]
		idx = 0
		lstate = 'idle'
		sbuf = b""
		lenCount = 0
		
		tlvCount = 0
		pbyte = 16
		v6 = ([])
		v7 = ([])
		v8 = ([])
	
		while True:
			try:
				ch = self.port.read()
			except:
				return (False,v6,v7,v8)
			#print(str(ch))
			if lstate == 'idle':
				#print(self.magicWord)
				if ch == self.magicWord[idx]:
					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
					idx += 1
					if idx == 8:
						idx = 0
						lstate = 'header'
						rangeProfile = b""
						sbuf = b""
				else:
					idx = 0
					rangeProfile = b""
					return (False,v6,v7,v8)
		
			elif lstate == 'header':
				sbuf += ch
				idx += 1
				if idx == 44:
					#print("------header-----")
					#print(":".join("{:02x}".format(c) for c in sbuf))  
					# [header - Magicword]
					try: 
						''' v0.0.3
						(self.hdr.version,self.hdr.platform,self.hdr.timestamp,
						self.hdr.totalPackLen,self.hdr.frameNumber,self.hdr.subframeNumber,
						self.hdr.chirpMargin, self.hdr.frameMargin,self.hdr.uartSendTime,
						self.hdr.trackProcessTime,self.hdr.numTLVs,
						self.hdr.checksum) = struct.unpack('6I4L2H', sbuf)
						'''
						# v0.0.4
						(self.hdr.version,self.hdr.platform,self.hdr.timestamp,
						self.hdr.totalPackLen,self.hdr.frameNumber,self.hdr.subframeNumber,
						self.hdr.chirpMargin, self.hdr.frameMargin,self.hdr.uartSendTime,
						self.hdr.trackProcessTime,self.hdr.numTLVs,
						self.hdr.checksum) = struct.unpack('10I2H', sbuf)
						
					except:
						if self.dbg == True:
							print("(Header)Improper TLV structure found: ")
						return (False,v6,v7,v8)
					
					if disp == True:  
						self.headerShow()
					
					tlvCount = self.hdr.numTLVs
					if self.hdr.numTLVs == 0:
						return (True,v6,v7,v8)
						
					if self.sm == True:
						print("(Header)")
						
					sbuf = b""
					idx = 0
					lstate = 'TL'
					
					  
				elif idx > 44:
					idx = 0
					lstate = 'idle'
					return (False,v6,v7,v8)
					
			elif lstate == 'TL':
				sbuf += ch
				idx += 1
				if idx == 8:
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try:
						ttype,self.tlvLength = struct.unpack('2I', sbuf)
						if ttype not in typeList or self.tlvLength > 10000:
							if self.dbg == True:
								print("(TL)Improper TL Length(hex):(T){:d} (L){:x} numTLVs:{:d}".format(ttype,self.tlvLength,self.hdr.numTLVs))
							sbuf = b""
							idx = 0
							lstate = 'idle'
							self.port.flushInput()
							return (False,v6,v7,v8)
							
					except:
						if self.dbg == True:
							print("TL unpack Improper Data Found:")
						self.port.flushInput()
						return (False,v6,v7,v8)
					
					lstate ,plen ,lenCount,self.numOfPoints = self.tlvTypeInfo(ttype,self.tlvLength,disp)
					
					if self.sm == True:
						print("(TL)=>({:})".format(lstate))
						
					tlvCount -= 1
					idx = 0  
					sbuf = b""
				
					 
			elif lstate == 'V6': # count = Total Lentgh - 8
				sbuf += ch
				idx += 1
				if (idx%plen == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf))
						(range2d,azimuth,doppler,snr) = struct.unpack('4f', sbuf)
						#print("range2d:{:.4f} azimuth:{:.4f} doppler:{:.4f} snr:{:.4f}".format(range2d,azimuth,doppler,snr))
						v6.append((range2d,azimuth,doppler,snr))
						#print("point_cloud_2d.append:[{:d}]".format(len(point_cloud_2d)))
						sbuf = b""
					except:
						if self.dbg == True:
							print("(6)Improper Type 6 Value structure found: ")
						return (False,v6,v7,v8)
					
				if idx == lenCount:
					if disp == True:
						print("v6[{:d}]".format(len(v6)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						return (True,v6,v7,v8)
						
					else: # Go to TL to get others type value
						lstate = 'TL' #'tlTL'
						if self.sm == True:
							print("(V6)=>(TL)")
					
				elif idx > lenCount:
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v6,v7,v8)
				
			elif lstate == 'V7':
				sbuf += ch
				idx += 1
				if (idx%plen == 0):
					try:
						EC = [0,0,0,0,0,0,0,0,0]
						(tid,posX,posY,velX,velY,accX,accY,EC[0],EC[1],EC[2],EC[3],EC[4],EC[5],EC[6],EC[7],EC[8],G) = struct.unpack('1I16f', sbuf)
						v7.append((tid,posX,posY,velX,velY,accX,accY,EC,G))
						sbuf = b""
					except:
						if self.dbg == True:
							print("(7)Improper Type 7 Value structure found: ")
						return (False,v6,v7,v8)
						
				if idx == lenCount:
					if disp == True:
						print("v7[{:d}]".format(len(v7)))
					idx = 0 
					sbuf = b""
					if tlvCount <= 0:
						lstate = 'idle'
						return (True,v6,v7,v8)
						
					else: # Go to TL to get others type value
						lstate = 'TL' 
						if self.sm == True:
							print("(V7)=>(TL)")

				if idx > lenCount:
					idx = 0 
					lstate = 'idle'
					sbuf = b""
					return (False,v6,v7,v8)
				
			elif lstate == 'V8':
				idx += 1
				v8.append(ord(ch))
				if idx == lenCount:
					if disp == True:
						print("v8:{:}".format(v8))
						print("=====V8 End====")
						
					sbuf = b""
					idx = 0
					lstate = 'idle'
					if self.sm == True:
						print("(V8)=>(idle)")
					return (True,v6,v7,v8)
				
				if idx > lenCount:
					sbuf = b""
					idx = 0
					lstate = 'idle'
					return (False,v6,v7,v8)







# Vital Sign Detect(VSD) key/value data
# ver:0.0.1
# date: 2020/10/20
# parsing Vital Sign Detect(VSD) key/value data
# hardware:(Batman-201): ISK IWR6843 ES2.0
# company: Joybien Technologies: www.joybien.com
# author: Zach Chen
#===========================================
# output: (header,h,flow,BR,HR,BP,HP,status,tail)
# v0.0.1 : 2020/10/20

import serial
import time
import struct
from dataclasses import dataclass

#import numpy as np

@dataclass
class info:
	h: chr = '{'
	f: chr = '0'
	BR: float = 0.0
	HR: float = 0.0
	BP: float = 0.0
	HP: float = 0.0
	s : chr = '0'
	t:chr = '}'


class VitalSign_kv:
	#				{        }  
	magicWord = [b'\x7b',b'\x7d'] 
	idata = info
	port = ""
	clen = 20
	
	def __init__(self,port):
		self.port = port
		print("(jb)Vital Sign Detect (VSD) key/value lib initial")
		print("(jb)For Hardware:Batman-201(ISK)")
		print("(jb)Hardware: IWR-6843 ES2.0")
		print("(jb)Firmware: VSD")
		print("(jb)UART Baud Rate:115200")
		print("(jb)Data type: kv (key/Value)")
		print("==============Info=================")
		print("Output: (header,h,flow,BR,HR,BP,HP,status,tail)")
		print("===================================")
	
	def tlvRead(self,disp):
		idx = 0
		lstate = 'idle'
		sbuf = b""
		while True:
			try:
				ch = self.port.read()
			except:
				print("(VSD)---port.read() Exception---")
				return (False, [])
			#print(str(ch))
			if lstate == 'idle':
				#print(self.magicWord)
				if ch == self.magicWord[0]:
					
					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
					idx = 0
					lstate = 'iData'
					sbuf = ch
					if disp:
						print("(jb) idle-> idata")
					
				else:
					idx = 0
					sbuf = b""
					return (False,[])
					
			elif lstate == 'iData':
				sbuf += ch
				#print(":".join("{:02x}".format(c) for c in sbuf))  
				idx += 1
				#print("ch: idx = {:02x} : {:d}    mg:{:02x}   bool:{:}".format(ord(ch),idx ,ord(self.magicWord[1]) , idx == 19 and self.magicWord[1] == ch))
				if idx == 19: # and self.magicWord[1] == ch:  #wierd 
					#print(":".join("{:02x}".format(c) for c in sbuf))  
					#idata = struct.unpack('<2b4f2b',sbuf) #(h,f,BR,HR,BP,HP,s,t)
					idata = struct.unpack('<cB4fBc',sbuf) #(h,f,BR,HR,BP,HP,s,t)
					if disp:
						print("(jb) iData-> idle")
					lstate = 'idle'
					sbuf = b""
					idx = 0
					return (True,idata) 
				if idx > 20:
					lstate = 'idle'
					sbuf = b""
					idx = 0
			



# Vehicle Occupancy Detection(VOD) and Child Presence Detection(CPD)
# for only
# ver:0.0.1
# 2020/02/10
# parsing Vehicle Occupancy Detection and Child Presence Detection(CPD) data 
# Use: HeatMap
# hardware:(Batman-301)VOD/CPD AWR6843
# company: Joybien Technologies: www.joybien.com
# author: Zach Chen
#===========================================
# output: V6,V7,V8 Raw data
# v0.0.1 : 2020/07/21 release

import serial
import time
import struct
#import numpy as np

class header:
	totalPackLen = 0
	platform = 0
	frameNumber = 0
	timeCpuCycles = 0
	numDetectedObj = 0
	numTLVs = 0


class VehicleODHeatMap:
	
	magicWord =  [b'\x02',b'\x01',b'\x04',b'\x03',b'\x06',b'\x05',b'\x08',b'\x07',b'\0x99']
	port = ""
	hdr = header
	 
	# add for VOD interal use
	tlvLength = 0
	
	# for debug use 
	dbg = False #Packet unpacket Check: True show message 
	sm = False  #Observed StateMachine: True Show message
	check = False # Observe numTLV and v length
	
	def __init__(self,port):
		self.port = port
		print("(jb)Vehicle Occupancy Detection(VOD) and Child Presence Detection(CPD)")
		print("(jb)For Hardware:AWR6843")
		print("(jb)SW:0.0.1: HW:IWR-1642")
		print("(jb)Firmware: VOD")
		print("(jb)UART Baud Rate:921600")
		print("(jb)==================================")
		print("(jb)Output: V8,V9,V10 data:(RAW)")
		print("(jb)V8 :Range Azimuth Heatmap TLV")
		print("(jb)V9 :Feature Vector TLV")
		print("(jb)V10:Decision Vector TLV")
		
		
	def checkTLV(self,ft):
		self.check = ft
		
	def useDebug(self,ft):
		self.dbg = ft
		
	def stateMachine(self,ft):
		self.sm = ft
		
	def getVitalSign(self):
		return self.vs
	
	def getHeader(self):
		return self.hdr
		
	def headerShow(self):
		print("***header***********") 
		#print("Version:     \t%x "%(self.hdr.version))
		print("Platform:    \t%X "%(self.hdr.platform))
		print("TotalPackLen:\t%d "%(self.hdr.totalPackLen))
		print("PID(frame#): \t%d "%(self.hdr.frameNumber))
		print("timeCpuCycles: \t%d "%(self.hdr.timeCpuCycles))
		print("numDetectedObj: \t%d "%(self.hdr.numDetectedObj))
		print("numTLVs: \t%d "%(self.hdr.numTLVs))
		#print("subFrameIndex: \t%d "%(self.hdr.subFrameIndex))
		print("***End Of Header***") 
		
		
		
	def tlvTypeInfo(self,dtype,count,dShow):
		
		dataByte = 0
		lenCount = count
		pString = ""
		stateString = "V8"
		if dtype == 8:
			lenCount = count
			dataByte= 2    #HeatMap Value Int:2 bytes float:4 bytes
			pString = "Range-Azimuth Heat Map"
			stateString = 'V8'
			
		elif dtype == 9:
			lenCount = count
			dataByte = 20  #target struct 20 bytes:(avgPower1,avgPower2,powerRatio1,powerRatio2,crossCorr)  
			pString = "Feature Vector TLV"
			stateString = "V9"
			
		elif dtype == 10:
			lenCount = count
			dataByte = 12 #zone = 12 byte (%:4,power:4,rangeIdx:2,AzmuthIdx:2)
			pString = "Decision Vector TLV"
			stateString = "V10"
			
		else:
			pString = "*** Type Error ***"
			stateString = 'idle'

		if dShow == True:
			print("-----[{:}] ----".format(pString))
			print("tlv Type :  \t{:d}".format(dtype))
			print("tlv length:      \t{:d}".format(lenCount)) 
			#print("{:}      \t{:d}".format(nString,int(nPoint)))
			#print("value length:    \t{:d}".format(retCnt))  
		
		return stateString,dataByte,lenCount,pString
		
#
# TLV: Type-Length-Value
# read TLV data
# input:
#     disp: True:print message
#			False: hide printing message
# output:(return parameter)
# (pass_fail, v8, v9, v10)
#  pass_fail: True: Data available    False: Data not available
#
#	Output: V8,V9,V10 data:(RAW)")
#	V8 :Range Azimuth Heatmap TLV 
#	V9 :Feature Vector TLV 
#	V10:Decision Vector TLV 
#

	def tlvRead(self,disp):
		#print("---tlvRead---")
		#ds = dos
		typeList = [8,9,10]
		idx = 0
		lstate = 'idle'
		sbuf = b""
		lenCount = 0
		unitByteCount = 0
		dataBytes = 0
		
		tlvCount = 0
		pbyte = 16
		v8 = []
		v9 = ([])
		v10 = ([])
		
		zone = 0
	
		while True:
			try:
				ch = self.port.read()
			except:
				return (False,v8,v9,v10)
			#print(str(ch))
			if lstate == 'idle':
				#print(self.magicWord)
				if ch == self.magicWord[idx]:
					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
					idx += 1
					if idx == 8:
						idx = 0
						lstate = 'header'
						rangeProfile = b""
						sbuf = b""
				else:
					#print("not: magicWord state:")
					idx = 0
					rangeProfile = b""
					return (False,v8,v9,v10)
		
			elif lstate == 'header':
				sbuf += ch
				idx += 1
				if idx == 24: 
					#print(":".join("{:02x}".format(c) for c in sbuf)) 	 
					#print("len:{:d}".format(len(sbuf))) 
					# [header - Magicword]
					try: 
						(self.hdr.totalPackLen,self.hdr.platform,
						self.hdr.frameNumber,self.hdr.timeCpuCycles,self.hdr.numDetectedObj,
						self.hdr.numTLVs) = struct.unpack('6I', sbuf)
					except:
						if self.dbg == True:
							print("(Header)Improper TLV structure found: ")
						return (False,v8,v9,v10)
					
					if disp == True:  
						self.headerShow()
					
					tlvCount = self.hdr.numTLVs
					if self.hdr.numTLVs == 0:
						return (False,v8,v9,v10)
						
					if self.sm == True:
						print("(Header)")
						
					sbuf = b""
					idx = 0
					lstate = 'TL'
					 
					  
				elif idx > 40:
					print("Header State over 40")
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v8,v9,v10)
					
			elif lstate == 'TL': #TLV Header type/length
				sbuf += ch
				idx += 1
				if idx == 8:
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try:
						ttype,self.tlvLength = struct.unpack('2I', sbuf)
						#if self.check:
						#	print("(check) numTLVs({:d}): tlvCount({:d})-------ttype:tlvLength:{:d}:{:d}".format(self.hdr.numTLVs,tlvCount,ttype,self.tlvLength))
						if ttype not in typeList or self.tlvLength > 10000:
							if self.dbg == True:
								print("(TL)Improper TL Length(hex):(T){:d} (L){:x} numTLVs:{:d}".format(ttype,self.tlvLength,self.hdr.numTLVs))
							sbuf = b""
							idx = 0
							lstate = 'idle'
							self.port.flushInput()
							return (False,v8,v9,v10)
							
					except:
						if self.dbg == True:
							print("TL unpack Improper Data Found:")
						self.port.flushInput()
						return (False,v8,v9,v10)
					
					lstate,dataBytes,lenCount,pString = self.tlvTypeInfo(ttype,self.tlvLength,disp)
					print()
					if self.sm == True:
						print("(TL:{:d})=>({:})".format(tlvCount,lstate))
						
					tlvCount -= 1
					idx = 0  
					sbuf = b""
			
					
			elif lstate == 'V8': # count = Total Lentgh - 8
				sbuf += ch
				idx += 1
				if (idx%dataBytes == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf))
						v8s = struct.unpack('h', sbuf)
						v8.append(v8s)
						sbuf = b""
					except: 
						if self.dbg == True:
							print("(6.1)Improper Type V8 structure found: ")
						return (False,v8,v9,v10)
					
				if idx == lenCount:
					if disp == True:
						print("v8[{:d}]".format(len(v8)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle' 
						if self.sm == True:
							print("(V8:{:d})=>(idle) :true".format(tlvCount))
						return (True,v8,v9,v10)
						
					else: # Go to TL to get others type value
						lstate = 'TL'  
						#print("---------v8 ok ---Goto:TL------{:d}".format(len(v8)))
						if self.sm == True:
							print("(V8:{:d})=>(TL)".format(tlvCount))
					
				elif idx > 10000: #lenCount:
					print("V8 data over:10")
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v8,v9,v10)
					
			elif lstate == 'V9':
				idx += 1
				sbuf += ch 
				#print("V9: idx({:d} lenCount:{:d} tlvCount({:d}".format(idx,lenCount,tlvCount))
				if (idx%dataBytes == 0):
					#idx = 0
					#avgPower1,avgPower2,powerRatio1,powerRatio1,crossCorr = struct.unpack('5f',sbuf)
					v9s =  struct.unpack('5f',sbuf)
					v9.append(v9s)
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						if self.sm == True:
							print("(V9:{:d})=>(idle) :true".format(tlvCount))
						return (True,v8,v9,v10) #zzzzz
					'''
					else:
						if self.sm == True:
							print("(V9)=>(TL)")
						lstate = 'TL'
					'''
				if idx == lenCount:
					if disp == True:
						print("v9[{:d}]".format(len(v9)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle' 
						if self.sm == True:
							print("(V9:{:d})=>(idle) :true".format(tlvCount))
						return (True,v8,v9,v10)
						
					else: # Go to TL to get others type value
						lstate = 'TL'  
						#print("---------v9 ok ---Goto:TL------{:d}".format(len(v9)))
						if self.sm == True:
							print("(V9:{:d})=>(TL)".format(tlvCount))
					
				if idx > lenCount:
					print("************** v9 idx > lenCount ******************  ")
					idx = 0 
					sbuf = b""
					lstate = 'idle'
					
			elif lstate == 'V10':
				idx += 1
				sbuf += ch
				#print("V10:idx={:d}".format(idx))
				#v10.append(ord(ch))
				if idx%dataBytes == 0:
					try:
						#print("V10:lenCount({:d})  idx:{:d}  dataBytes={:d}".format(lenCount,idx,dataBytes))
						#v10.append((percent,power,rangeIdx,azimuthIdx))
						v10s = struct.unpack('2f2H', sbuf)
						v10.append(v10s)
						'''
						print("************* v10s ************ v10.len({:d})".format(len(v10)))
						print(v10s)
						print("************* v10s end *********** ") 
						'''
						sbuf = b""
						if disp == True:
							print("v10[{:d}]".format(len(v10)))
					except:
						print("(V10)Improper Type 10 structure found")
						return (False,v8,v9,v10)
				
				if idx == lenCount:
					idx = 0
					sbuf = b""
					if tlvCount <= 0:
						lstate = 'idle'
						if self.sm == True:
							print("(V10)=>(idle) :true")
						return (True,v8,v9,v10)
					
					'''
					else: # Go to TL to get others type value
						lstate = 'TL'
						idx = 0
						sbuf = b""
						#print("(V10)=>(TL) len={:d}".format(len(v10)))
						if self.sm == True:
							print("(V10)=>(TL)")
					'''
				if idx > lenCount:
					print("V10 data over:lenCount:({:d})".format(lenCount))
					idx = 0 
					lstate = 'idle'
					sbuf = b""
					if self.sm == True:
						print("(V10)=>(idle)")
					return (False,v8,v9,v10)
			



# Surface Velocity Detection  Radar
# for only
# ver:0.0.1
# 2020/04/15
# parsing  Surface Velocity Detection Sensor 
# hardware:(Batman-201)VOD IWR6843
# company: Joybien Technologies: www.joybien.com
# author: Zach Chen
#===========================================
# output: V1,V2,V6,V7 Raw data
# v0.0.1 : 2020/04/17 release

import serial
import time
import struct
#import numpy as np
from dataclasses import dataclass

class header:
	version = 0
	totalPackLen = 0
	platform = 0
	frameNumber = 0
	timeCpuCycles = 0
	numDetectedObj = 0
	numTLVs = 0
	subFrameIndex = 0
	
class v1Class:
	x = 0.0
	y = 0.0
	z = 0.0
	velocity = 0.0
	
@dataclass	
class statsInfo:
	interFrameProcessingTime = 0
	transmitOutputTime = 0
	interFrameProcessingMargin = 0
	interChirpProcessingMargin = 0
	activerFrameCPULoad = 0
	interFrameCPULoad = 0
	
class SurfaceVD:
	
	magicWord =  [b'\x02',b'\x01',b'\x04',b'\x03',b'\x06',b'\x05',b'\x08',b'\x07',b'\0x99']
	port = ""
	hdr = header
	
	dck = 0
	# add for VOD interal use
	tlvLength = 0
	
	# for debug use 
	dbg = False #Packet unpacket Check: True show message 
	sm = False #Observed StateMachine: True Show message
	check = False # Observe numTLV and v length
	
	def __init__(self,port):
		self.port = port
		print("(jb)Surface Velocity Detection")
		print("(jb)For Hardware:IWR1642")
		print("(jb)SW:0.0.1: HW:IWR-1642")
		print("(jb)Firmware: SVD")
		print("(jb)UART Baud Rate:921600")
		print("(jb)==================================")
		print("(jb)Output: V1,V2,V6,V7 data:(RAW)")
		print("(jb)V1 :Point Cloud TLV")
		print("(jb)V2 :Range Profile TLV")
		print("(jb)V6 :Stats Information TLV")
		print("(jb)V7 :Point Cloud Side Info TLV")
		
	def checkTLV(self,ft):
		self.check = ft
		
	def useDebug(self,ft):
		self.dbg = ft
		
	def stateMachine(self,ft):
		self.sm = ft
		
	def getVitalSign(self):
		return self.vs
	
	def getHeader(self):
		return self.hdr
		
	def headerShow(self):
		print("***header***********") 
		print("Version:     \t%x "%(self.hdr.version))
		print("Platform:    \t%X "%(self.hdr.platform))
		print("TotalPackLen:\t%d "%(self.hdr.totalPackLen))
		print("PID(frame#): \t%d "%(self.hdr.frameNumber))
		print("timeCpuCycles: \t%d "%(self.hdr.timeCpuCycles))
		print("numDetectedObj: \t%d "%(self.hdr.numDetectedObj))
		print("numTLVs: \t%d "%(self.hdr.numTLVs))
		print("subFrameIndex: \t%d "%(self.hdr.subFrameIndex))
		print("***End Of Header***") 
		
		
		
	def tlvTypeInfo(self,dtype,count,dShow):
		
		dataByte = 0
		lenCount = count
		pString = ""
		stateString = "V1"
		if dtype == 1:
			dataByte= 16  
			pString = "(V1)Point Cloud"
			lenCount = count
			stateString = 'V1'
			
		elif dtype == 2:
			lenCount = count
			dataByte = 2  #target struct 2 bytes:(fft size)  
			pString = "(V2)Range Profile"
			stateString = "V2"
		elif dtype == 6:
			lenCount = count
			dataByte = 24  #target struct 24 bytes:(ifpT,toT,ifpM,icpM,afcL,ifCL)  
			pString = "(V6)Stats Info"
			stateString = "V6"	
		elif dtype == 7:
			lenCount = count
			dataByte = 4  #target struct 4 bytes:(snr,noise)  
			pString = "(V7)Point Cloud Side Info"
			stateString = "V7"
	
		else:
			pString = "*** Type Error ***"
			stateString = 'idle'

		if dShow == True:
			print("-----[{:}] ----".format(pString))
			print("tlv Type :  \t{:d}".format(int(dtype)))
			print("tlv length:      \t{:d}".format(int(lenCount))) 
			#print("{:}      \t{:d}".format(nString,int(nPoint)))
			#print("value length:    \t{:d}".format(retCnt))  
		
		return stateString,dataByte,lenCount,pString
		
#
# TLV: Type-Length-Value
# read TLV data
# input:
#     disp: True:print message
#			False: hide printing message
# output:(return parameter)
# (pass_fail, v1, v2, v6,v7)
#  pass_fail: True: Data available    False: Data not available
#
#	Output: V1,V2,V6,V7 data:(RAW)")
#	V1 :Point Cloud TLV
#	V2 :Range Profile TLV
#	V6 :Stats Information TLV
#	V7 :Point Cloud Side Info TLV

	def tlvRead(self,disp):
		#print("---tlvRead---")
		
		typeList = [1,2,6,7] # zzzz
		idx = 0
		lstate = 'idle'
		sbuf = b""
		lenCount = 0
		unitByteCount = 0
		dataBytes = 0
		
		tlvCount = 0
		pbyte = 16
		v1 = []
		v2 = []
		v6 = []
		v7 = []
		
		zone = 0
	
		while True:
			try:
				ch = self.port.read()
			except:
				return (False,v1,v2,v6,v7)
			#print(str(ch))
			if lstate == 'idle':
				#print(self.magicWord)
				if ch == self.magicWord[idx]:
					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
					idx += 1
					if idx == 8:
						idx = 0
						lstate = 'header'
						rangeProfile = b""
						sbuf = b""
				else:
					#print("not: magicWord state:")
					idx = 0
					rangeProfile = b""
					return (False,v1,v2,v6,v7)
		
			elif lstate == 'header':
				sbuf += ch
				idx += 1
				if idx == 32: # 32 + 12  = 44
					#print("------header-----")
					#print(":".join("{:02x}".format(c) for c in sbuf)) 	 
					#print("len:{:d}".format(len(sbuf))) 
					# [header - Magicword]
					try: 
						(self.hdr.version,self.hdr.totalPackLen,self.hdr.platform,
						self.hdr.frameNumber,self.hdr.timeCpuCycles,self.hdr.numDetectedObj,
						self.hdr.numTLVs,self.hdr.subFrameIndex) = struct.unpack('8I', sbuf)
					
					except:
						if self.dbg == True:
							print("(Header)Improper TLV structure found: ")
						return (False,v1,v2,v6,v7)
					
					if disp == True:  
						self.headerShow()
					
					tlvCount = self.hdr.numTLVs
						
					if self.hdr.numTLVs == 0 or self.hdr.numTLVs > 10:
						lstate = 'idle'
						return (False,v1,v2,v6,v7)
						
					if self.sm == True:
						print("(Header)")
						
					sbuf = b""
					idx = 0
					lstate = 'TL'
					dck = 0
					  
				elif idx > 45:
					idx = 0
					lstate = 'idle'
					return (False,v1,v2,v6,v7)
					
			elif lstate == 'TL': #TLV Header type/length
				sbuf += ch
				idx += 1
				if idx == 8:
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try:
						ttype,self.tlvLength = struct.unpack('2I', sbuf)
						if self.check:
							print("(check) numTLVs({:d}): tlvCount({:d})---ttype:{:d} tlvLength:{:d}".format(self.hdr.numTLVs,tlvCount,ttype,self.tlvLength))
						if ttype not in typeList:
							if self.dbg == True:
								print("(TL)Improper TL Length(hex):(T){:d} (L){:x} numTLVs:{:d}".format(ttype,self.tlvLength,self.hdr.numTLVs))
							sbuf = b""
							idx = 0
							lstate = 'idle'
							self.port.flushInput()
							return (False,v1,v2,v6,v7)
						if  self.tlvLength > self.hdr.totalPackLen:
							sbuf = b""
							idx = 0
							lstate = 'idle'
							self.port.flushInput()
							return (False,v1,v2,v6,v7)
							
					except:
						if self.dbg == True:
							print("TL unpack Improper Data Found:")
						self.port.flushInput()
						return (False,v1,v2,v6,v7)
					
					lstate,dataBytes,lenCount,pString = self.tlvTypeInfo(ttype,self.tlvLength,disp)
					if lstate == 'V10':
						self.zone = self.tlvLength
					 
					if self.sm == True:
						print("(TL:{:d})=>({:})".format(tlvCount,lstate))
						
					tlvCount -= 1
					idx = 0  
					sbuf = b""
					
			elif lstate == 'V1': #zzzz
				sbuf += ch
				idx += 1
				#print(".".join("{:02x}".format(c) for c in sbuf))
				if (idx%dataBytes == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf))
						v1s = struct.unpack('4f', sbuf)
						v1.append(v1s)
						#print(v1)
						sbuf = b''
					except:
						if self.dbg == True:
							print("(V1)Improper Type V1 structure found: ")
						return (False,v1,v2,v6,v7)
				
				if idx == lenCount:
					if disp == True:
						print("V1[{:d}]".format(len(v1)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						dck += 1
						if self.sm == True:
							print("(V1:{:d})=>(idle) :true".format(tlvCount))
						return (False,v1,v2,v6,v7)
						
					else: # Go to TL to get others type value
						lstate = 'TL' 
						dck += 1
						idx = 0
						sbuf = b""
						if self.sm == True:
							print("(V1:{:d})=>(TL)".format(tlvCount))
						
							
				elif idx > self.hdr.totalPackLen: #lenCount:
					print("V1 data over:{:d} totalPackLen:{:d}".format(idx,self.hdr.totalPackLen))
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v1,v2,v6,v7)
						 
			elif lstate == 'V7': # zzzz
				sbuf += ch
				idx += 1
				#print(":".join("{:02x}".format(c) for c in sbuf))
				if (idx%dataBytes == 0):
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try:	
						v7s = struct.unpack('2h', sbuf)						
						v7.append(v7s)
						sbuf = b""
					except:
						if self.dbg == True:
							print("(6.1)Improper Type V7 structure found: ")
						return (False,v1,v2,v6,v7)
					
				if idx == lenCount:
					if disp == True:
						print("v7[{:d}]".format(len(v7)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						dck += 1
						if self.sm == True:
							print("(V7:{:d})=>(idle) :true".format(tlvCount))
						return (True,v1,v2,v6,v7)
						
					else: # Go to TL to get others type value
						#print(v7)
						lstate = 'TL' 
						dck += 1
						idx = 0
						sbuf = b""
						#print("---------v7 ok ---------{:d}".format(len(v8)))
						if self.sm == True:
							print("(V7:{:d})=>(TL)".format(tlvCount))
					
				elif idx > self.hdr.totalPackLen: #lenCount:
					print("V8 data over:10")
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v1,v2,v6,v7)
			
			elif lstate == 'V2': # zzzz
				sbuf += ch
				idx += 1
				#print(".".join("{:02x}".format(c) for c in sbuf))
				if (idx%dataBytes == 0):
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try:	
						v2s = struct.unpack('<H', sbuf)	
						v2f = float(v2s[0]) / 512				
						v2.append(v2f)
						sbuf = b""
					except:
						print("(6.1)Improper Type V2 structure found: ")
						if self.dbg == True:
							print("(6.1)Improper Type V2 structure found: ")
						return (False,v1,v2,v6,v7)
					
				if idx == lenCount:
					if disp == True:
						print("v2[{:d}]".format(len(v2)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						dck += 1
						if self.sm == True:
							print("(V2:{:d})=>(idle) :true".format(tlvCount))
						return (True,v1,v2,v6,v7)
						
					else: # Go to TL to get others type value
						#print(v2)
						lstate = 'TL' 
						dck += 1
						idx = 0
						sbuf = b""
						#print("---------v7 ok ---------{:d}".format(len(v8)))
						if self.sm == True:
							print("(V2:{:d})=>(TL)".format(tlvCount))
					
				elif idx > self.hdr.totalPackLen: #lenCount:
					print("V2 data over:10")
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v1,v2,v6,v7)	
			
			elif lstate == 'V6': # zzzz
				sbuf += ch
				idx += 1
				#print(".".join("{:02x}".format(c) for c in sbuf))
				if (idx%dataBytes == 0):
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try:		
						v6s = struct.unpack('<6I', sbuf)		
						v6.append(v6s)
						sbuf = b""
						#print(v6)
					except:
						print("(6.1)Improper Type V6 structure found: ")
						if self.dbg == True:
							print("(6.1)Improper Type V6 structure found: ")
						return (False,v1,v2,v6,v7)
					
				if idx == lenCount:
					if disp == True:
						print("v6[{:d}]".format(len(v6)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						dck += 1
						if self.sm == True:
							print("(V6:{:d})=>(idle) :true".format(tlvCount))
						return (True,v1,v2,v6,v7)
						
					else: # Go to TL to get others type value
						#print(v6)
						lstate = 'TL' 
						dck += 1
						idx = 0
						sbuf = b""
						#print("---------v7 ok ---------{:d}".format(len(v8)))
						if self.sm == True:
							print("(V6:{:d})=>(TL)".format(tlvCount))
					
				elif idx > self.hdr.totalPackLen: #lenCount:
					print("V6 data over:10")
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v1,v2,v6,v7)				
					
					
			elif lstate == 'V8': # count = Total Lentgh - 8
				sbuf += ch
				idx += 1
				if (idx%dataBytes == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf))
						v8s = struct.unpack('h', sbuf)						
						v8.append(v8s[0])
						sbuf = b""
					except:
						if self.dbg == True:
							print("(6.1)Improper Type V8 structure found: ")
						return (False,v8,v9,v10,v11)
					
				if idx == lenCount:
					if disp == True:
						print("v8[{:d}]".format(len(v8)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						dck += 1
						if self.sm == True:
							print("(V6:{:d})=>(idle) :true".format(tlvCount))
						return (True,v8,v9,v10,v11)
						
					else: # Go to TL to get others type value
						lstate = 'TL' 
						dck += 1
						idx = 0
						sbuf = b""
						#print("---------v8 ok ---------{:d}".format(len(v8)))
						if self.sm == True:
							print("(V8:{:d})=>(TL)".format(tlvCount))
					
				elif idx > self.hdr.totalPackLen: #lenCount:
					print("V8 data over:10")
					idx = 0
					sbuf = b""
					lstate = 'idle'
					return (False,v8,v9,v10,v11)
					
			elif lstate == 'V9':
				idx += 1
				sbuf += ch
				 
				if idx == lenCount: 
					#avgPower1,avgPower2,powerRatio1,powerRatio1,crossCorr = struct.unpack('5f',sbuf)
					v9 =  struct.unpack('5f',sbuf)
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						if self.sm == True:
							print("(V9:{:d})=>(idle) :true".format(tlvCount))
						return (True,v8,v9,v10,v11)
					else:
						if self.sm == True:
							print("(V9)=>(TL)")
						lstate = 'TL'
						
				if idx > lenCount:
					idx = 0 
					sbuf = b""
					lstate = 'idle'
					
			elif lstate == 'V10':
				idx += 1
				v10.append(ord(ch))
				if idx == lenCount:
					#print("V10:dataBytes({:d}) lenCount({:d}) index:{:d}".format(dataBytes,lenCount,idx))
					if disp == True:
						print("v10[{:d}]".format(len(v10)))
					 
					sbuf = b""
					if tlvCount <= 0:
						lstate = 'idle'
						if self.sm == True:
							print("(V10)=>(idle) :true")
						return (True,v8,v9,v10,v11)
						
					else: # Go to TL to get others type value
						lstate = 'TL'
						idx = 0
						sbuf = b""
						#print("(V10)=>(TL) len={:d}".format(len(v10)))
						if self.sm == True:
							print("(V10)=>(TL)")

				if idx > 10:
					print("V10 data over:10")
					idx = 0 
					lstate = 'idle'
					sbuf = b""
					if self.sm == True:
						print("(V10)=>(idle)")
					return (False,v8,v9,v10,v11)
			
			elif lstate == 'V11': 
				#print("dataBytes:{:d}  lenCount:{:d}   idx:{:d}".format(dataBytes,lenCount,idx))
				sbuf += ch
				idx += 1
				if idx%dataBytes == 0:
					#unwrapped,heart,breathing,heart_rate,breathing_rate = struct.unpack('5f',sbuf)
					#v11.append((unwrapped,heart,breathing,heart_rate,breathing_rate))
					self.vs = struct.unpack('5f',sbuf)
					v11.append(self.vs)
					#print("uw:{:f} heart:{:f} breathing:{:f}  hrate:{:f}  bRate:{:f} ".format(unwrapped,heart,breathing,heart_rate,breathing_rate))
					if disp == True:
						print("(V11)")
					sbuf = b"" 
				
				if idx == lenCount:
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						if self.check:
							print("V11 ----> idle")
						#print("(V11)len = {:d}".format(len(v11)))
						dck += 8
						if self.sm == True:
							print("(V11:{:d})=>(idle) :true".format(tlvCount))
						return (True,v8,v9,v10,v11)
					else:
						lstate =  'TL'
						if self.sm == True:
							print("(V11)=>(TL)")
						 
						
				if idx > 60:
					print("V11 data over:60")
					if self.sm == True:
							print("(V11)=>(TL)")
					idx = 0 
					lstate = 'idle'
					sbuf = b""
					return (False,v8,v9,v10,v11)
			







# lpd FDS(Falling Detection Sensing)  - ISK Raw Data 
# ver:0.0.1
# 2021/05/24
# 
# parsing lpd FDS -ISK
# hardware:(Batman-201)ISK IWR6843 ES2.0 
#    Fireware version: I471
# config file: 
# company: Joybien Technologies: www.joybien.com
# author: Zach Chen
#===========================================
# output: V6,V7,V8,V9 Raw data & dataFrame
# v0.0.1 : 2020/06/19 release
#          (1)Output list data
# v0.1.0 : 2021/05/024 
#          (1)Output DataFrame
# v0.1.1 : 2021/05/26
# v0.1.2 : 2021/05/26 bug fix
#
import serial
import time
import struct
import pandas as pd
import numpy as np

class header:
	version = 0
	platform = 0
	timeStamp = 0
	totalPackLen = 0
	frameNumber = 0
	subframeNumber = 0
	chirpMargin = 0
	frameMargin = 0 
	trackProcessTime = 0
	uartSendTime = 0
	numTLVs = 0
	checksum = 0


class LpdFDS:
	
	magicWord =  [b'\x02',b'\x01',b'\x04',b'\x03',b'\x06',b'\x05',b'\x08',b'\x07',b'\0x99']
	port = ""
	hdr = header
	
	# provide csv file dataframe
	# real-time 
	v6_col_names_rt = ['fN','type','range','azimuth','elv','doppler','sx', 'sy', 'sz']
	v7_col_names_rt = ['fN','type','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','ec0','ec1','ec2','ec3','ec4','ec5','ec6','ec7','ec8','ec9','ec10','ec11','ec12','ec13','ec14','ec15','g','confi','tid']
	v8_col_names_rt = ['fN','type','targetID']
	v9_col_names_rt = ['fN','type','snr','noise']
	# read from file for trace point clouds
	fileName = ''
	v6_col_names = ['time','fN','type','range','azimuth','elv','doppler','sx', 'sy', 'sz']
	v7_col_names = ['time','fN','type','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','ec0','ec1','ec2','ec3','ec4','ec5','ec6','ec7','ec8','ec9','ec10','ec11','ec12','ec13','ec14','ec15','g','confi','tid']
	v8_col_names = ['time','fN','type','targetID']
	v9_col_names = ['time','fN','type','snr','moise']
	sim_startFN = 0
	sim_stopFN  = 0 
	
	# add for interal use
	tlvLength = 0
	numOfPoints = 0
	# for debug use 
	dbg = False #Packet unpacket Check: True show message 
	sm  = False #Observed StateMachine: True Show message
	plen = 16 
	
	def __init__(self,port):
		self.port = port
		print("(jb)lpd Falling Detection Sensing initial")
		print("(jb)vsersion:v0.1.0")
		print("(jb)For Hardware:Batman-201(ISK)")
		print("(jb)Hardware: IWR-6843")
		print("(jb)Firmware: lpd-I471")
		print("(jb)UART Baud Rate:921600")
		print("Output: V6,V7,V8,V9 data:(RAW)")
		
	def useDebug(self,falseTrue):
		self.dbg = falseTrue
		
	def stateMachine(self,falseTrue):
		self.sm = falseTrue
		
	def getHeader(self):
		return self.hdr
		
		
	def headerShow(self):
		print("******* Header ********") 
		print("Version:     \t%x "%(self.hdr.version))
		print("Platform:    \t%X "%(self.hdr.platform))
		print("TimeStamp:    \t%X "%(self.hdr.timeStamp))
		print("TotalPackLen:\t%d "%(self.hdr.totalPackLen))
		print("PID(frame#): \t%d "%(self.hdr.frameNumber))
		print("subframe#  : \t%d "%(self.hdr.subframeNumber))
		print("Inter-frame Processing Time:\t{:d} us".format(self.hdr.trackProcessTime))
		print("UART Send Time:\t{:d} us".format(self.hdr.uartSendTime))
		print("Inter-chirp Processing Margin:\t{:d} us".format(self.hdr.chirpMargin))
		print("Inter-frame Processing Margin:\t{:d} us".format(self.hdr.frameMargin))
		print("numTLVs:     \t%d "%(self.hdr.numTLVs))
		print("Check Sum   :\t{:x}".format(self.hdr.checksum))
		print("***End Of Header***") 
			
	#for class internal use
	def tlvTypeInfo(self,dtype,count,dShow):
		
		sbyte = 8  #tlvHeader Struct = 8 bytes
		
		dataByte = 0
		pString = ""
		nString = "numOfPoints :"
		stateString = "V6"
		if dtype == 6:
			
			sbyte = 0      #tlvHeader Struct = 8 bytes (type(4)/lenth(4))    0 bytes: not counting
			dataByte= 16    #pointStruct 8bytes:(range(4),azimuth(4),elevation(4),doppler(4))
			pString = "Point Cloud Spherical"
			nString = ""
			stateString = "V6"
		elif dtype == 7:
			
			sbyte = 0	   #tlvHeader Struct = 8 bytes (type(4)/lenth(4))    0 bytes: not counting
			dataByte = 112  #target struct 112 bytes:(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0,ec1,ec2,ec3,ec4,ec5,ec6,ec7,ec8,ec9,ec10,ec11,ec12,ec13,ec14,ec15,g,confi)  
			pString = "Target Object TLV"
			nString = "numOfObjects:"
			stateString = "V7"
		
		elif dtype == 8:
			sbyte = 0    #tlvHeader Struct = 8 bytes  0 bytes: not counting
			dataByte = 1 #targetID = 1 byte
			pString = "Target Index TLV"
			nString = "numOfIDs"
			stateString = "V8"
		elif dtype == 9:
			
			sbyte = 0    #tlvHeader Struct = 8 bytes  0 bytes: not counting
			dataByte = 4 #snr(2 bytes),noise(2 bytes) = 4 bytes
			pString = "Point Cloud Side Info TLV"
			nString = ""
			stateString = "V9"
		else:
			tlvTypeInfo
			sbyte = 1
			pString = "*** Type Error ***"
			stateString = 'idle'
		 
		retCnt = count - sbyte
		nPoint = retCnt / dataByte
		#dShow = True
		if dShow == True:
			print("-----[{:}] ----".format(pString))
			print("tlv Type({:2d}Bytes):  \t{:d}".format(sbyte,dtype))
			print("tlv length:      \t{:d}".format(count)) 
			print("{:}      \t{:d}".format(nString,int(nPoint)))
			print("value length:    \t{:d}".format(retCnt))  
		
		return stateString, sbyte, dataByte,retCnt, int(nPoint)
		
	def list2df(self,dck,l6,l7,l8,l9):
		ll6 = pd.DataFrame(l6,columns=self.v6_col_names_rt)
		ll7 = pd.DataFrame(l7,columns=self.v7_col_names_rt)
		ll8 = pd.DataFrame(l8,columns=self.v8_col_names_rt)
		ll9 = pd.DataFrame(l9,columns=self.v9_col_names_rt)
		return (dck,ll6,ll7,ll8,ll9)

#
# TLV: Type-Length-Value
# read TLV data
# input:
#     disp: True:print message
#			False: hide printing message
#     df: 
# output:(return parameter)
# (pass_fail, v6, v7, v8, v9)
#  pass_fail: True: Data available    False: Data not available
#  v6: point cloud infomation
#  v7: Target Object information
#  v8: Target Index information
#  v9: Point Cloud Side Information
# 
#
	def tlvRead(self,disp,df = None):
		
		#print("---tlvRead---")
		#ds = dos
		typeList = [6,7,8,9]
		idx = 0
		lstate = 'idle'
		sbuf = b""
		lenCount = 0
		unitByteCount = 0
		dataBytes = 0
		numOfPoints = 0
		tlvCount = 0
		pbyte = 16
		v6 = ([])
		v7 = ([])
		v8 = ([])
		v9 = ([])
		v6df = ([])
		v7df = ([])
		v8df = ([])
		v9df = ([])
		
		while True:
			try:
				ch = self.port.read()
			except:
				return self.list2df(False,v6,v7,v8,v9) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
				
			#print(str(ch))
			if lstate == 'idle':
				#print(self.magicWord)
				if ch == self.magicWord[idx]:
					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
					idx += 1
					if idx == 8:
						idx = 0
						lstate = 'header'
						rangeProfile = b""
						sbuf = b""
				else:
					#print("not: magicWord state:")tlvTypeInfo
					idx = 0
					rangeProfile = b""
					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
		
			elif lstate == 'header':
				sbuf += ch
				idx += 1
				if idx == 44:  
					#print("------header-----")
					#print(":".join("{:02x}".format(c) for c in sbuf)) 	 
					#print("len:{:d}".format(len(sbuf))) 
					# [header - Magicword]
					try: 
						(self.hdr.version,self.hdr.platform,
						self.hdr.timeStamp,self.hdr.totalPackLen,
						self.hdr.frameNumber,self.hdr.subframeNumber,
						self.hdr.chirpMargin,self.hdr.frameMargin,self.hdr.trackProcessTime,self.hdr.uartSendTime,
						self.hdr.numTLVs,self.hdr.checksum) = struct.unpack('10I2H', sbuf)
						self.frameNumber = self.hdr.frameNumber
					except:
						if self.dbg == True:
							print("(Header)Improper TLV structure found: ")
						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
					
					if disp == True:  
						self.headerShow()
					
					tlvCount = self.hdr.numTLVs
					#print("tlvCount:{:}".format(tlvCount))
					if self.hdr.numTLVs == 0:
						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
						
					if self.sm == True:
						print("(Header)")
						
					sbuf = b""
					idx = 0
					lstate = 'TL'
					  
				elif idx > 44:
					idx = 0
					lstate = 'idle'
					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
					
			elif lstate == 'TL': #TLV Header type/length
				sbuf += ch
				idx += 1
				if idx == 8:
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try:
						ttype,self.tlvLength = struct.unpack('2I', sbuf)
						if disp == True:
							print("(TL)--tlvNum:{:d}: tlvCount({:d})-------ttype:tlvLength:v{:d}:{:d}".format(self.hdr.numTLVs,tlvCount,ttype,self.tlvLength))
							
						if ttype not in typeList or self.tlvLength > 10000:
							if self.dbg == True:
								print("(TL)Improper TL Length(hex):(T){:d} (L){:x} numTLVs:{:d}".format(ttype,self.tlvLength,self.hdr.numTLVs))
							sbuf = b""
							idx = 0
							lstate = 'idle'
							self.port.flushInput()
							return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
							
					except:
						if self.dbg == True:
							print("TL unpack Improper Data Found:")
						self.port.flushInput()
			
						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
					
					lstate ,plen ,dataBytes,lenCount, numOfPoints = self.tlvTypeInfo(ttype,self.tlvLength,disp)
					#if ttype == 6:
					#	print("--pointCloud:((tlvLength({:d})-pointUnit(20)-tlvStruct(8))/8={:d}".format(self.tlvLength,numOfPoints))
					if self.sm == True:
						print("(TL:{:d})=>({:})".format(tlvCount,lstate))
						
					tlvCount -= 1
					idx = 0  
					sbuf = b""
			
					
			elif lstate == 'V6': # count = Total Lentgh - 8
				sbuf += ch
				idx += 1
				#print("v6 dataBytes:{:}  idx:{:} , lenCount:{:}".format(dataBytes,idx,lenCount))
				if (idx%dataBytes == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf
						#print("V6: sbuf length:{:}".format(len(sbuf)))
						(r,a,e,d) = struct.unpack('4f', sbuf)
						elv = e 
						azi = a 
						dop = d 
						ran = r 
						#print("v6:length:{:}  elv:{:.4f} azimuth:{:.4f} doppler:{:.4f} range:{:.4f}".format(len(v6),elv,azi,dop,ran))
						
						if (df == 'DataFrame'):
							sz  = ran * np.sin(elv)
							sx  = ran * np.cos(elv) * np.sin(azi)
							sy  = ran * np.cos(elv) * np.cos(azi)
							v6df.append((self.hdr.frameNumber,'v6',ran,azi,elv,dop,sx,sy,sz)) # v ok
							 
						else:
							v6.append((ran,azi,elv,dop))
						
						#print("point_cloud_2d.append:[{:d}]".format(len(point_cloud_2d)))
						sbuf = b""
						
					except:
						if self.dbg == True:
							print("(6.1)Improper Type 6 Value structure found: ")
						#return (False,v6,v7,v8)
						#return (False,v6,v7,v8) if (df == None) else self.list2df(False,v6df,v7df,v8df)
						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
					
				if idx == lenCount:
					if disp == True:
						print("v6[{:d}]".format(lenCount))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						if self.sm == True:
							print("(V6:{:d})=>(idle) :true".format(tlvCount))
						#return (True,v6,v7,v8)
						#return (True,v6,v7,v8) if (df == None) else self.list2df(True,v6df,v7df,v8df)
						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
						
					else: # Go to TL to get others type value
						lstate = 'TL' #'tlTL'
						if self.sm == True:
							print("(V6:{:d})=>(TL)".format(tlvCount))
					
				elif idx > lenCount:
					idx = 0
					sbuf = b""
					lstate = 'idle'
					#return (False,v6,v7,v8)
					#return (False,v6,v7,v8) if (df == None) else self.list2df(False,v6,v7,v8)
					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
					
			 
			elif lstate == 'V9':
				sbuf += ch
				idx += 1
				#print("v9 dataBytes:{:}  idx:{:} , lenCount:{:}".format(dataBytes,idx,lenCount))
				if (idx%dataBytes == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf))
						(snr,noise) = struct.unpack('2H', sbuf)
						
						#print("v9: snr:{:.4f} noise:{:.4f}".format(snr,noise))
						
						if (df == 'DataFrame'):
							v9df.append((self.hdr.frameNumber,'v9',snr,noise))
						else:
							v9.append((snr,noise))
						
						sbuf = b""
						
					except:
						if self.dbg == True:
							print("(9.1)Improper Type 9 Value structure found: ")
							
						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
					
				if idx == lenCount:
					if disp == True:
						print("v9[{:d}]".format(len(v9)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						if self.sm == True:
							print("(V9:{:d})=>(idle) :true".format(tlvCount))
							
						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
						
					else: # Go to TL to get others type value
						lstate = 'TL' #'tlTL'
						if self.sm == True:
							print("(V9:{:d})=>(TL)".format(tlvCount))
					
				elif idx > lenCount:
					idx = 0
					sbuf = b""
					lstate = 'idle'
					#return (False,v6,v7,v8)
					#return (False,v6,v7,v8) if (df == None) else self.list2df(False,v6,v7,v8)
					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
			 
			elif lstate == 'V7':
				sbuf += ch
				idx += 1
				#print("v7 dataBytes:{:}  idx:{:} , lenCount:{:}".format(dataBytes,idx,lenCount))
				 
				if (idx%dataBytes == 0):
					#print("V7:dataBytes({:d}) lenCount({:d}) index:{:d}".format(dataBytes,lenCount,idx))
					try:
						(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0,ec1,ec2,ec3,ec4,ec5,ec6,ec7,ec8,ec9,ec10,ec11,ec12,ec13,ec14,ec15,g,confi) = struct.unpack('I27f', sbuf)
						if (df == 'DataFrame'): 
							v7df.append((self.hdr.frameNumber,'v7',posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0,ec1,ec2,ec3,ec4,ec5,ec6,ec7,ec8,ec9,ec10,ec11,ec12,ec13,ec14,ec15,g,confi,tid))
						else:
							v7.append((tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0,ec1,ec2,ec3,ec4,ec5,ec6,ec7,ec8,ec9,ec10,ec11,ec12,ec13,ec14,ec15,g,confi))
						
						sbuf = b""
					except:
						if self.dbg == True:
							print("(7)Improper Type 7 Value structure found: ")
						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
						
				if idx >= lenCount:
					if disp == True:
						print("v7[{:d}]".format(len(v7)))
					idx = 0
					sbuf = b""
					if tlvCount == 0:
						lstate = 'idle'
						if self.sm == True:
							print("(V7)=>(idle) :true")
						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
						
					else: # Go to TL to get others type value
						lstate = 'TL'
						idx = 0
						if self.sm == True:
							print("(V7)=>(TL)")

				if idx > lenCount:
					idx = 0 
					lstate = 'idle'
					sbuf = b""
					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
				
			elif lstate == 'V8':
				idx += 1
				v8.append(ord(ch))
				if (df == 'DataFrame'):
					v8df.append((self.hdr.frameNumber,'v8',ord(ch)))
				
				if idx == lenCount:
					if disp == True:
						print("=====V8 End===={:}".format(v8))
					sbuf = b""
					idx = 0
					lstate = 'idle'
					if self.sm == True:
						print("(V8:{:d})=>(idle)".format(tlvCount))
					return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
				
				if idx > lenCount:
					sbuf = b""
					idx = 0
					lstate = 'idle'
					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
					
					
			
			
					
	def v67Simlog(frameNum):
		global sim_startFN,sim_stopFN
		s_fn = frameNum + sim_startFN
		#print("frame number:{:}".format(s_fn))
		v6d = v6sim[v6sim['fN'] == s_fn]
		#v6d =  v6dd[v6dd['doppler'] < 0.0]
		#print(v6d)
		v7d = v7sim[v7sim['fN'] == s_fn]
		chk = 0
		if v6d.count != 0:
			chk = 1
		return (chk,v6d,v7d)
		
	def getRecordData(self,frameNum):
		s_fn = frameNum + self.sim_startFN
		#print("frame number:{:}".format(s_fn))
		v6d = self.v6simo[self.v6simo['fN'] == s_fn]
		#v6d =  v6dd[v6dd['doppler'] < 0.0]
		#print(v6d)
		v7d = self.v7simo[self.v7simo['fN'] == s_fn]
		v8d = self.v8simo[self.v8simo['fN'] == s_fn]
		chk = 0
		if v6d.count != 0:
			chk = 1
		return (chk,v6d,v7d,v8d)
		
	def readFile(self,fileName):
		#fileName = "pc32021-03-19-10-02-17.csv"  
		#df = pd.read_csv(fileName, error_bad_lines=False, warn_bad_lines=False) 
		self.fileName = fileName 
		#          ['time','fN','type','elv','azimuth','range' ,'doppler','snr','sx', 'sy', 'sz']
		df = pd.read_csv(self.fileName, names = self.v6_col_names, skiprows = [0,11,12]) 
		df.dropna()
		#print("------------------- df --------------------shape:{:}".format(df.shape))
		print(df.info())
		print(df.info(memory_usage="deep")) 
		
		v6simOri = df[(df.type == 'v6')]
		#print("-------------------v6sim------------:{:}".format(v6simOri.shape))
		#self.v6simo = v6simOri.loc[:,['fN','elv','azimuth','range','doppler','snr']]
		self.v6simo = v6simOri.loc[:,['fN','type','elv','azimuth','range' ,'doppler','snr','sx', 'sy', 'sz']]
		self.v6simo['elv'] = self.v6simo['elv'].astype(float, errors = 'raise') 
		
		df7 = pd.read_csv(self.fileName, names = self.v7_col_names, skiprows = [0])  
		
		v7simc = df7[df7['type'] == 'v7']
		self.v7simo  = v7simc.loc[:,['fN','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','ec0','ec1','ec2','ec3','ec4','ec5','ec6','ec7','ec8','ec9','ec10','ec11','ec12','ec13','ec14','ec15','g','confi','tid']]
		self.sim_startFN = df['fN'].values[0]
		self.sim_stopFN  = df['fN'].values[-1]
		'''
		print("---------start frame number---------:{:}".format(self.sim_startFN))
		print("---------stop  frame number---------:{:}".format(self.sim_stopFN))
		print(self.v7simo)
		print("-----v8sim data lib-----")
		'''
		v8simc = df[df['type'] == 'v8']
		self.v8simo  = v8simc.loc[:,['fN','type','elv']]
		self.v8simo.columns = ['fN','type','targetID']
		#print(self.v8simo)
		return (self.v6simo,self.v7simo,self.v8simo)





# Traffic Monitor Detector Raw Data (TMD_I480)
# modify from TMD
# v6,v7,v8,v9
#
# parsing Traffic Monitor Detector
# hardware:(Batman-201/601): ISK IWR6843/1843
# company: Joybien Technologies: www.joybien.com
# author: Zach Chen
# ver:0.1.1 2021/09/16
# ver:2.0.1 2022/04/28
# ver:2.0.2 2022/05/12
# ver:2.0.3 2022/05/12
# ver:2.0.4 2022/05/15
#===========================================
# output: V6,V7,V8,v9 Raw data
#
# playback data type:
# 	v6: list/dataFrame
# 	v7: list/dataFrame
# 	v8: dictionary
# 	v9: dictionary


import serial
import time
import struct
import pandas as pd
import numpy as np
import csv

class header:
	version = 0
	totalPackLen = 0
	platform = 0
	timeStamp = 0
	totalPacketLen = 0
	frameNumber = 0
	subFrameNumber = 0
	chirpMargin = 0
	frameMargin = 0 
	trackProcessTime = 0
	uartSendTime = 0
	numTLVs = 0
	checksum = 0


class TrafficMD_I480:
	
	magicWord =  [b'\x01',b'\x02',b'\x03',b'\x04',b'\x05',b'\x06',b'\x07',b'\x08',b'\0x99']
	port = ""
	hdr = header

	# provide csv file dataframe
	# real-time 
	v6_col_names_rt = ['fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz'] #ok
	v7_col_names_rt = ['fN','type','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','tid']
	v8_col_names_rt = ['fN','type','targetID']
	v9_col_names_rt = ['fN','type','snr','noise']
	
	# read from file for trace point clouds
	fileName = ''
	v6_col_names = ['time','fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']
	v7_col_names = ['time','fN','type','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','tid']
	v8_col_names = ['time','fN','type','targetID']
	v9_col_names = ['time','fN','type','snr','noise']
	sim_startFN = 0
	sim_stopFN  = 0 
	
	v6simo = []
	v7simo = []
	v8simo = {}
	v9simo = {}
	
	# add for interal use
	tlvLength = 0
	numOfPoints = 0
	# for debug use 
	dbg = False #Packet unpacket Check: True show message 
	sm  = False #Observed StateMachine: True Show message
	plen = 16 
	
	def __init__(self,port):
		self.port = port
		print("(jb)Traffic Monitor Detector(TMD_I480) raw Data lib initial")
		print("(jb)For Hardware:Batman-201/501/601(ISK)")
		print("(jb)Version: v0.1.0")
		print("(jb)Hardware: IWR-6843/1843")
		print("(jb)Firmware: TMD")
		print("(jb)UART Baud Rate:921600")
		print("==============Info=================")
		print("Output: V6,V7,V8,V9 data:(RAW)")
		print("V6: Point Cloud Spherical")
		print("V6 structure: [(range,azimuth,elevation,doppler,sx,sy,sz),......]")
		print("V7: Target Object List")
		print("V7 structure: [(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ),....]")
		print("V8: Target Index")
		print("V8 structure: [id1,id2....]")
		print("V9: Point Cloud Side Info")
		print("V9 [(snr,noise),....]")
		print("")
		print("tlvRead status: {0:'EMPTY',1:'inData',10:'IDLE',99:'FALSE'} ")
		print("")
		print("===================================")
	 
		
	def useDebug(self,ft):
		self.dbg = ft
		
	def stateMachine(self,ft):
		self.sm = ft
		
	def getHeader(self):
		return self.hdr
	
	#################################################################
	# for version manegement
	def jb_u32ToBytes(self, v): # convert u32 as Bytes
		A = np.zeros( (4, 1) )
		for i in range(4):
			A[3 - i] = int(v  % 256)
			v = int(v / 256)
		return int(A[0, 0]), int(A[1, 0]), int(A[2, 0]), int(A[3, 0])
	
	def jb_versionManagement(self):
		# test, for example: 
		# result as following,
		# (1) Platform    : IWR ISK, JB, 1843
		# (2) FW   version: V80.61R
		# (3) TOOL version: I.04.08.00
		#
		# (1) Platform
		vHardware, vMagic, vType, vRxTxNum = self.jb_u32ToBytes(self.hdr.platform)			
		# hardware type
		if int(vHardware / 16) == 0:
			tmp = ''
		elif int(vHardware / 16) == 1:
			tmp = 'IWR '
		elif int(vHardware / 16) == 2:
			tmp = 'AWR '
		else:
			tmp = ''
		# hardware antenna
		if int(vHardware % 16) == 0:
			tmp += ''
		elif int(vHardware % 16) == 1:
			tmp += 'ISK'
		elif int(vHardware % 16) == 2:
			tmp += 'AOP'
		else:
			tmp += ''
		vHardware = tmp	
		# sel magic
		if int(vMagic % 16) == 15:
			vMagic = 'JB'
		else:
			vMagic = ''			
		print("Platform    : {}, {}, {:02x}{:02x}".format(vHardware, vMagic, vType, vRxTxNum))
		# (2) FW version
		vAnt, vMagic, vMajor, vMinor = self.jb_u32ToBytes(self.hdr.chirpMargin)			
		# hardware antenna
		if vAnt == 'I':
			vAnt = ', Ant= ISK'
		elif vAnt == 'A':
			vAnt = ', Ant= AOP'
		else:	
			vAnt = ''
		print("FW   version: V{:02d}.{:02d}{:c} {}".format(vMajor, vMinor, vMagic, vAnt))
		# (3) TOOL version
		vTool, vMajor, vMinor, vMicro = self.jb_u32ToBytes(self.hdr.frameMargin)			
		print("TOOL version: {:c}.{:02d}.{:02d}.{:02d}".format(vTool, vMajor, vMinor, vMicro))
	#################################################################
				
	def headerShow(self):
		print("***Header***********") 
		print("Version:     \t%x "%(self.hdr.version))
		#print("Platform:    \t%X "%(self.hdr.platform))
		print("TimeStamp:   \t%s "%(self.hdr.timeStamp))
		print("TotalPackLen:\t%d "%(self.hdr.totalPackLen))
		print("PID(frame#): \t%d "%(self.hdr.frameNumber))
		print("subframe#  : \t%d "%(self.hdr.subFrameNumber))
	    ########################################################
		# original
		#print("Platform:    \t%X "%(self.hdr.platform))		
		#print("Inter-chirp Processing Margin:\t{:d} us".format(self.hdr.chirpMargin))
		#print("Inter-frame Processing Margin:\t{:d} us".format(self.hdr.frameMargin))
		# new replaced
		self.jb_versionManagement()
	    ########################################################
		
		print("Inter-frame Processing Time:\t{:d} us".format(self.hdr.trackProcessTime))
		print("UART Send Time:\t{:d} us".format(self.hdr.uartSendTime))
		print("numTLVs:     \t%d "%(self.hdr.numTLVs))
		print("Check Sum   :\t{:x}".format(self.hdr.checksum))
		print("***End Of Header***") 
			
	#for class internal use
	def tlvTypeInfo(self,dtype,count,dShow):
		sbyte = 8  #tlvHeader Struct = 8 bytes
		unitByte = 20 
		dataByte = 0
		pString = ""
		nString = "numOfPoints :"
		stateString = "V6"
		if dtype == 6:
			unitByte = 0   #pointUnit= 20bytes
			sbyte = 8      #tlvHeader Struct = 8 bytes
			dataByte= 16    #pointStruct 8bytes:(range,azimuth,elevation,doppler)
			pString = "DPIF Point Cloud Spherical TLV"
			nString = ""
			stateString = "V6"
		elif dtype == 7:
			unitByte = 0   #pointUnit= 0bytes 
			sbyte = 8 	   #tlvHeader Struct = 8 bytes
			#old: dataByte = 40  #target struct 40 bytes:(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ)  # I441
			dataByte = 112 #target struct 112 bytes:(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0..15,g,c) # I480 
			pString = "Target Object List TLV"
			nString = "numOfObjects:"
			stateString = "V7"
		elif dtype == 8:
			unitByte = 0 #pointUnit= 0bytes 
			sbyte = 8    #tlvHeader Struct = 8 bytes
			dataByte = 1 #targetID = 1 byte
			pString = "Target Index TLV"
			nString = "numOfIDs"
			stateString = "V8"
		elif dtype == 9:
			unitByte = 0 #pointUnit= 0bytes 
			sbyte = 8    #tlvHeader Struct = 8 bytes
			dataByte = 4 #(snr,noise") =  4 byte
			pString = "DPIF Point Cloud Side Info"
			nString = "numOfIDs"
			stateString = "V9"
		else:
			unitByte = 0
			sbyte = 1
			pString = "*** Type Error ***"
			stateString = 'idle'
		 
		#retCnt = count - unitByte - sbyte
		nPoint = count / dataByte
		if dShow == True:
			print("-----[{:}] ----:{:}".format(pString,stateString))
			print("tlv Type({:2d}Bytes):  \t{:d}".format(sbyte,dtype))
			print("tlv length:      \t{:d}".format(count)) 
			print("num of point:    \t{:d}".format(int(nPoint)))
			print("value length:    \t{:d}".format(count))  
		
		return unitByte,stateString, sbyte, dataByte,count, int(nPoint)
		
	def list2df(self,dck,l6,l7,l8,l9):
		#print("---------------list2df: v6----------------")
		#print(l6)
		
		ll6 = pd.DataFrame(l6,columns=self.v6_col_names_rt)
		ll7 = pd.DataFrame(l7,columns=self.v7_col_names_rt)
		ll8 = pd.DataFrame(l8,columns=self.v8_col_names_rt)
		ll9 = pd.DataFrame(l9,columns=self.v9_col_names_rt)
		#print("------ll6---------list2df: v6----------------")
		#print(ll6)
		return (dck,ll6,ll7,ll8,ll9)

#
# TLV: Type-Length-Value
# read TLV data
# input:
#     disp: True:print message
#			False: hide printing message
# output:(return parameter)
# (pass_fail, v6, v7, v8, v9)
#  pass_fail: True: Data available    False: Data not available
#
#  v6: DPIF point cloud Spherical infomation
#  v7: Target Object List information
#  v8: Target Index information
#  v9: DPIF Point Cloud Side Infomation
#  
#   0: empty: tlv = 0
#   1: data output
#  
#   10: idle
#   99: error
#
	def tlvRead(self,disp,df = None):
		#print("---tlvRead---")
		#ds = dos
		typeList = [6,7,8,9]
		idx = 0
		lstate = 'idle'
		sbuf = b""
		lenCount = 0
		unitByteCount = 0
		dataBytes = 0
		numOfPoints = 0
		tlvCount = 0
		pbyte = 16
		v6 = ([])
		v7 = ([])
		v8 = ([])
		v9 = ([])
		v6df = ([])
		v7df = ([])
		v8df = ([])
		v9df = ([])
	
		while True:
			try:
				ch = self.port.read()
			except:
				#return (False,v6,v7,v8,v9)
				return self.list2df(99,v6,v7,v8,v9) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
			#print(str(ch))
			if lstate == 'idle':
				#print(self.magicWord)
				if ch == self.magicWord[idx]:
					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
					idx += 1
					if idx == 8:
						idx = 0
						lstate = 'header'
						rangeProfile = b""
						sbuf = b""
				else:
					#print("not: magicWord state:")
					idx = 0
					rangeProfile = b""
					#return (False,v6,v7,v8,v9)
					return self.list2df(10,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (10,v6,v7,v8,v9)
		
			elif lstate == 'header':
				sbuf += ch
				idx += 1
				if idx == 44: 
					#print("------header-----")
					#print(":".join("{:02x}".format(c) for c in sbuf)) 	 
					#print("len:{:d}".format(len(sbuf))) 
					# [header - Magicword]
					try: 
						(self.hdr.version,self.hdr.platform,self.hdr.timeStamp,self.hdr.totalPackLen,
						self.hdr.frameNumber,self.hdr.subFrameNumber,
						self.hdr.chirpMargin,self.hdr.frameMargin,self.hdr.trackProcessTime,self.hdr.uartSendTime,
						self.hdr.numTLVs,self.hdr.checksum) = struct.unpack('10I2H', sbuf)
						
					except:
						if self.dbg == True:
							print("(Header)Improper TLV structure found: ")
						#return (False,v6,v7,v8,v9)
						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
					
					if disp == True:  
						self.headerShow()
					
					tlvCount = self.hdr.numTLVs
					if self.hdr.numTLVs == 0:
						#return (False,v6,v7,v8,v9)
						#return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
						return self.list2df(0,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (0,v6,v7,v8,v9)
						
					if self.sm == True:
						print("(Header)")
						
					sbuf = b""
					idx = 0
					lstate = 'TL'
					  
				elif idx > 48:
					idx = 0
					lstate = 'idle'
					#return (False,v6,v7,v8,v9)
					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
					
			elif lstate == 'TL': #TLV Header type/length
				sbuf += ch
				idx += 1
				if idx == 8:
					#print(":".join("{:02x}".format(c) for c in sbuf))
					try: 
						ttype,self.tlvLength = struct.unpack('2I', sbuf)
						#print("(TL)numTLVs({:d}): tlvCount({:d})-------ttype:tlvLength:{:d}:{:d}".format(self.hdr.numTLVs,tlvCount,ttype,self.tlvLength))
						
						if ttype not in typeList or self.tlvLength > 10000:
							if self.dbg == True:
								print("(TL)Improper TL Length(hex):(T){:d} (L){:x} numTLVs:{:d}".format(ttype,self.tlvLength,self.hdr.numTLVs))
							sbuf = b""
							idx = 0
							lstate = 'idle'
							self.port.flushInput()
							#return (False,v6,v7,v8,v9)
							return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
							
					except:
						print("TL unpack Improper Data Found:")
						if self.dbg == True:
							print("TL unpack Improper Data Found:")
						self.port.flushInput()
						#return (False,v6,v7,v8,v9)
					
						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
					
					unitByteCount,lstate ,plen ,dataBytes,lenCount, numOfPoints = self.tlvTypeInfo(ttype,self.tlvLength,disp)
					#print("dataBytes={:d} lenCount={:d}".format(dataBytes , lenCount))
					if self.sm == True:
						print("(TL:{:d})=>({:})".format(tlvCount,lstate))
						
					tlvCount -= 1
					idx = 0  
					sbuf = b""
			
			elif lstate == 'V6': # count = Total Lentgh - 8
				sbuf += ch
				idx += 1
				if (idx%dataBytes == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf))
						(r,a,e,d) = struct.unpack('4f', sbuf)
						#print("({:2d}:{:4d})(idx:({:4d}) elv:{:.4f} azimuth:{:.4f} doppler:{:.4f} range:{:.4f}".format(numOfPoints,lenCount,idx,e,a,d,r))
						sz  = r * np.sin(e)
						sx  = r * np.cos(e) * np.sin(a)
						sy  = r * np.cos(e) * np.cos(a)
						if (df == 'DataFrame'):
							
							#fieldNames = ['time','frameNum','type','range/px','azimuth/py','elv/vx','doppler/vy','sx/accX','sy/accY','sz/pz','na/vz','na/accZ','na/ID']
							v6df.append((self.hdr.frameNumber,'v6',r,a,e,d,sx,sy,sz))
						else:
							v6.append((r,a,e,d,sx,sy,sz))
							
						sbuf = b""
					except:
						if self.dbg == True:
							print("(6.1)Improper Type 6 Value structure found: ")
						#return (False,v6,v7,v8,v9)
						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
					
				if idx == lenCount:
					if disp == True:
						print("v6[{:d}]".format(len(v6)))
					idx = 0
					sbuf = b""
					if tlvCount <= 0: # Back to idle
						lstate = 'idle'
						if self.sm == True:
							print("(V6:tlvCnt={:d})=>(idle) :true".format(tlvCount))
						#return (True,v6,v7,v8,v9)
						print("(lib)v6=============SaveAppend==============")
						return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
						
					else: # Go to TL to get others type value
						lstate = 'TL' #'tlTL'
						if self.sm == True:
							print("(V6:tlvCnt={:d})=>(TL)".format(tlvCount))
					
				elif idx > lenCount:
					idx = 0
					sbuf = b""
					lstate = 'idle'
					#return (False,v6,v7,v8,v9)
					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
					
			elif lstate == 'V9':  
				sbuf += ch
				idx += 1
				if (idx%dataBytes == 0):
					try:
						#print(":".join("{:02x}".format(c) for c in sbuf))
						(snr,noise) =  struct.unpack('2h', sbuf)
						#print("lenCnt = ({:d}) snr:{:.4f} noise:{:.4f}".format(lenCount,snr,noise))
						
						if (df == 'DataFrame'):
							v9df.append((self.hdr.frameNumber,'v9',snr,noise))
						else:
							v9.append((snr,noise))
							
						sbuf = b""
					except:
						if self.dbg == True:
							print("(7)Improper Type 9 Value structure found: ")
						#return (False,v6,v7,v8,v9)
						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
						
				if idx >= lenCount:
					if disp == True:
						print("count= v9[{:d}]".format(len(v9)))
						
					sbuf = b""
					if tlvCount == 0:
						lstate = 'idle'
						if self.sm == True:
							print("(V9)=>(idle) :true")
						#return (True,v6,v7,v8,v9)
						return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
						
					else: # Go to TL to get others type value
						lstate = 'TL'
						idx = 0
						if self.sm == True:
							print("(V9)=>(TL)")
							
				if idx > lenCount:
					idx = 0 
					lstate = 'idle'
					sbuf = b""
					#return (False,v6,v7,v8,v9)
					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
					
			elif lstate == 'V7':
				sbuf += ch
				idx += 1
				if (idx % dataBytes == 0):
					
					#print("V7:dataBytes({:d}) lenCount({:d}) index:{:d}".format(dataBytes,lenCount,idx))
					try:
						
						(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0,ec1,ec2,ec3,ec4,ec5,ec6,ec7,ec8,ec9,ec10,ec11,ec12,ec13,ec14,ec15,g,conf) = struct.unpack('I27f', sbuf) 
						if (df == 'DataFrame' ):
							v7df.append((self.hdr.frameNumber,'v7',posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,tid))
						else:
							v7.append((posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,tid))
							#print("tid = ({:d}) ,posX:{:.4f} posY:{:.4f} posZ:{:.4f}".format(tid,posX,posY,posZ))
						sbuf = b""
					except:
						if self.dbg == True:
							print("(7)Improper Type 7 Value structure found: ")
						#return (False,v6,v7,v8,v9)
						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
						
				if idx >= lenCount:
					if disp == True:
						print("v7[{:d}]".format(len(v7)))
					 
					sbuf = b""
					if tlvCount == 0:
						lstate = 'idle'
						if self.sm == True:
							print("(V7)=>(idle) :true")
						#return (True,v6,v7,v8,v9)
						return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
						
					else: # Go to TL to get others type value
						lstate = 'TL'
						idx = 0
						if self.sm == True:
							print("(V7)=>(TL)")

				if idx > lenCount:
					idx = 0 
					lstate = 'idle'
					sbuf = b""
					#return (False,v6,v7,v8,v9)
					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
				
			elif lstate == 'V8':
				idx += 1
				v8.append(ord(ch))
				if idx == lenCount:
					if disp == True:
						print("=====V8 End====")
					sbuf = b""
					idx = 0
					lstate = 'idle'
					if self.sm == True:
						print("(V8:{:d})=>(idle)".format(tlvCount))
						
					if (df == 'DataFrame'):
						v8o = [self.hdr.frameNumber,'v8']
						v8df = v8o.extend(v8)
					#return (True,v6,v7,v8,v9)
					return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
				
				if idx > lenCount:
					sbuf = b""
					idx = 0
					lstate = 'idle'
					#return (False,v6,v7,v8,v9)
					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)

	
	############## for playback use #####################
	def readFile(self,fileName):
		with open(fileName, newline='') as csvfile:
			rows = csv.reader(csvfile)
			v6s = [];v7s = [];v8s = [];v9s = []
			for row in rows:
				if row[2] == 'v6':
					v6s.append(row)
				elif row[2] == 'v7':
					v7s.append(row)
				elif row[2] == 'v8':
					v8s.append(row)
				elif row[2] == 'v9':
					v9s.append(row)
					
		v6_col_names = ['time','fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']
		self.v6simo = pd.DataFrame(v6s,columns = v6_col_names).astype({'fN': 'int32','type':str,'range':float,'azimuth':float,'elv': float,'doppler':float,'sx':float,'sy':float,'sz':float},errors = 'raise') 
		
		if len(self.v6simo):
			#print(f"self.v6simo['fN'].values[0] = {self.v6simo['fN'].values[0]}")
			self.sim_startFN =  self.v6simo['fN'].values[0]
			self.sim_stopFN  =  self.v6simo['fN'].values[-1]
			
		v7_col_names = ['time','fN','type','posX','posY','posZ','velX','velY','velZ','accX','accY','accZ','tid'] 
		self.v7simo = pd.DataFrame(v7s,columns = v7_col_names )
		
		for item in v9s:
			hh = item[3].strip('[]').replace('(', "").replace(')','').split(',')
			self.v9simo[int(item[1])] = [(int(hh[i]),int(hh[i+1])) for i in range(0,len(hh),2)]
		
		for item in v8s:
			hh = item[3].strip('[]').split(',')
			self.v8simo[int(item[1])] = [int(hh[i]) for i in range(len(hh))] 
			

	def getRecordData(self,frameNum):
		s_fn = frameNum + self.sim_startFN
		v6d = self.v6simo.loc[self.v6simo['fN'] == s_fn]  #if (len(self.v6simo) > 0) else []
		v7d = self.v7simo.loc[self.v7simo['fN'] == s_fn]  #if (len(self.v7simo) > 0) else []
		try:
			v8d =  self.v8simo[s_fn]    
		except:
			v8d = {}
			
		try:
			v9d =  self.v9simo[s_fn] 
		except:
			v9d = {}
		 
		return(1,v6d,v7d,v8d,v9d)
	



# (1)   $export PATH=$PATH:$HOME/.local/bin
# (2)   $python3 setup.py sdist bdist_wheel
#
# run Twine to upload all of the archives under dist:
# (3)   $twine upload --repository-url https://upload.pypi.org/legacy/ dist/*
#  if file HTTPError: 400 Client Error: File already exists.
#
#  Use this ******
# (3.1) $twine upload --skip-existing dist/*
# (4)   $sudo pip install mmWave -U
#
# if error: invalid command 'bdist_wheel'
# $pip3 install wheel
#
#
# vitalSign.py v0.0.8
#              v0.0.9
#              v0.0.10 :2020/02/04
#
# highAccuracy v0.0.4
#       v0.0.4 fix for Jetson nano
#
# peopleMB : v0.0.3
#           v0.0.4    :2019/12/06  fix for Jetson nano
#           v0.0.5    :2020/04/21  fix for Jetson nano
#
# srradar.py : v0.0.1 :2019/10/03
# v0.1.19 : add srradar.py
#
# people3D.py : v0.0.2 :2019/10/22
#               v0.0.3 :2019/10/23
#               v0.0.4 :2019/10/24
#
# pc3d.py     : v0.0.1 :2019/11/26
#               v0.0.2 :2019/11/27
#               v0.0.3 :2019/12/02
#
# pc3d_kv.py  : v0.0.1 :2019/11/25
#               v0.0.2 :2019/12/02
#
# lpdISK.py   : v0.0.1 :2019/12/04
#
# lpdISK_kv.py: v0.0.1 :2019/12/06
#
# vehicleOD.py   : v0.0.1 :2020/02/11
#
# trafficMD_kv.py :v0.0.2 :2020/03/18
# Original name: trafficMD.py :v0.0.1 :2020/03/18
#
# surfaceVD.py :v0.0.1 :2020/04/17
#
# trafficMD.py :v0.0.1 :2020/04/30
#               v0.1.0 :2021/04/21 add DataFrame output @v1.0.48
#
# droneRD.py   :v0.0.1 :2020/05/13
# droneRN.py   :v0.0.2 :2020/05/13 (change name from droneRD.py)
#
# pc3.py       :v0.0.1 :2020/06/19     added for v0.1.42
#               v0.1.1 :2021/04/21     add DataFrame output @v1.0.48
#               v0.1.2 :2021/05/06     add frameNumber and field change @v1.0.51
#               v0.1.3 :2021/05/06     revised dataframe dtype @v1.0.52
#               v0.1.4 :2021/10/12     doppler and range swap
#
# zoneOD.py    :v0.0.1 :2020/07/21     added for v1.0.43 removed at v1.0.44
#
# vehicleODHeatMap.py :v0.0.1 :2020/07/21 add for v0.1.44
#
# vitalsign_kv.py :v0.0.1: 2020/10/20 
#
# vehicleODR.py :v0.0.1 :2021/01/07 v0.1.47
#
# roadwayTMD_kv :v0.1.0 :2021/04/21 @v0.1.49
#               :v0.1.1 :2020/05/02 @v0.1.50 add getRecordData(self,frameNum):
#                                                readFile(self,fileName):
#               :v0.1.2 :2020/05/04 @v0.1.51 bug fix
#
# pc3OVH        :v0.1.0 :2021/05/18
#               :v0.1.1 :2021/08/12
#               :v0.1.2 :2021/09/16 @v7 in dataFrame 'tid' move to front of 'ec0'
#
# lpdFDS        :v0.1.0 :2021/05/26 dataFrame/raw 
#               :v0.1.1 :2021/05/26 message change
#               :v0.1.2 :2021/05/26 bug fix
#
# trafficMD_I480.py :v0.1.0 :2021/09/16
#
#
# mrRadar.py    :v0.0.1 :2021/12/31 First release @v0.1.62
#               :v0.0.2 :2022/02/07 format revised @v0.1.63
#               :v0.1.0 :2022/06/02 tune to fit v3 data
#               :v1.0.0 :2022/06/08 change get Uart Data Method
#               :v1.0.1 :2022/06/09 added headerShowAll()
# pc3_oob.py    :v0.0.1 :2022/03/01 @v0.1.64
#	            :v0.0.2 :2022/03/01 @v0.1.65 bug fix
#               :v0.0.3 :2022/03/03 @v1.0.66
# @v1.0.67 modified description
#
# @v1.0.68
# pc3_vsd.py    :v0.0.1 :2022/03/10 bata version
#
# @v1.0.70
# pc3_v2.py     :v2.0 :2022/04/19
#
# @v1.0.71
# trafficMD_I480 :v2.0.1 :2022/04/28 (output data posZ,velZ,accZ position change)
#
# @v1.0.72 (remove)
# trafficMD_I480 :v2.0.2 added tlvRead status, chk: {0:'EMPTY',1:'inData',10:'IDLE',99:'FALSE'}
#
# @v1.0.73
# trafficMD_I480 :v2.0.2 added tlvRead status, chk: {0:'EMPTY',1:'inData',10:'IDLE',99:'FALSE'}
#
# @v1.0.74
# trafficMD_I480 :v2.0.4 revised v8,v9 as dictionary data type
#
# @v1.0.75
# mrRadar : modify for fit v3 data
#
# @v1.0.76
# mrRadar : change get Uart method
# @v1.0.77
# mrRadar : added headerShowAll()
#
# @v1.0.78
# pc3_v2: v2.0.1 added sx,sy,sz in v6
#
# @v1.0.79
# v2.0.2 2022/06/13 pc3_v2 add azimuth offset

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mmWave",
    version="0.1.79",
    author="Bighead Chen",
    author_email="zach_chen@joybien.com",
    description="Joybien mmWave (Batman-101/201/301/501/601) library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://www.joybien.com",
    download_url="https://pypi.org/project/mmWave",
    project_urls={
        "API Source Code": "https://github.com/bigheadG/mmWave",
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



#)Traffic Monitor Detect(TMD) key Value data
# ver:0.0.2
# date: 2020/04/30
# parsing Traffic Monitor Detect 
# hardware:(Batman-201): ISK IWR6843 ES2.0
# company: Joybien Technologies: www.joybien.com
# author: Zach Chen
#===========================================
# output: objPoint: [(tidx,posX,posY,velX,velY,iten,Tid),....]
# v0.0.1 : 2020/03/09 release
# v0.0.2 : 2020/04/30 change name to trafficMD_kv

import serial
import time
import struct
from dataclasses import dataclass

#import numpy as np

@dataclass
class subHeader:
	frame: int = 0 #unsign Long
	target: int = 0 #unsign Int
	pcNum : int = 0 #unsign Int Point Cloud Number

@dataclass
class objPoint:
	idx: int = 0
	x: float = 0.0
	y: float = 0.0
	vx: float = 0.0
	vy: float = 0.0
	iten : int = 0
	tid : int = 0


class tmdISK_kv:
	#				{        }        ,      ;
	magicWord =  [b'\x5B',b'\x5D',b'\x3C',b'\x3E']
	
	port = ""
	clen = 20 #exclude: delimeter
	
	def __init__(self,port):
		self.port = port
		print("(jb)Traffic Monit Detect(TMD) lib initial")
		print("(jb)For Hardware:Batman-201(ISK)")
		print("(jb)Hardware: IWR-6843 ES2.0")
		print("(jb)Firmware: TMD")
		print("(jb)UART Baud Rate:921600")
		print("(jb)Data type: kv (key/Value)")
		print("==============Info=================")
		print("Output: frameNum,TargetNum,pcNum")
		print("Output: [(tidx,posX,posY,velX,velY,intensity,Tid),....]")
		print("===================================")
	
	def tmdRead(self,disp):
		target = 0
		targetCnt = 0
		ops = []
		sh = []
		frameNum = 0
		targetCnt = 0
		objMax = 0
		idx = 0
		lstate = 'idle'
		sbuf = b""
		while True:
			try:
				ch = self.port.read()
			except:
				print("(TMD)---port.read() Exception---")
				return (False, [])
			#print(str(ch))
			if lstate == 'idle':
				#print(self.magicWord)
				if ch == self.magicWord[0]:
					#if disp:
					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
					idx = 0
					lstate = 'iData'
					ops = [] #Object Point Set
					sbuf = ch
					target = 0 
					#if disp:
					#	print("-----------------------")
					#	print("(jb) idle-> idata")
					
				else:
					idx = 0
					sbuf = b""
					ops = [] 
					return (False,[],[])
					
			elif lstate == 'iTarget':
				sbuf += ch
				idx += 1
				if self.magicWord[2] == ch:
					#print("(jb)iTarget -> iTarget_end")
					lstate = 'iTarget_end'
					idx = 0
					
				else:
					lstate = 'idle'
					sbuf = 0
					return (False,[],[])
					
			elif lstate == 'iTarget_end':
				sbuf += ch
				idx += 1
				#print(":".join("{:02x}".format(c) for c in sbuf))  
				if idx == 21 and self.magicWord[3] == ch:
					#print(":".join("{:02x}".format(c) for c in sbuf))   
					#print("(jb)iTarget_end state:")
					try: 
						(h,idx,x,y,vx,vy,tar,iten,t) = struct.unpack('<cB4fBHc', sbuf) 
						ops.append(objPoint(idx,x,y,vx,vy,tar,iten))
						lstate = 'iTarget'
						idx = 0
						sbuf = b""
						
					except:
						lstate = 'idle'
						print("(jb)Exception ==> iTarget_end -> idle state ")
						return (False,sh,ops)
					
					target -= 1
					if target == 0:
						lstate = 'idle'
						shdr = []
						idx = 0
						sbuf = b""
						return (True,sh,ops)
				
				elif idx > 23:
					lstate = 'idle'
					sbuf = b""
					print("*********data over(iTarget_end)*********")
		
			elif lstate == 'iData':
				sbuf += ch
				#print(":".join("{:02x}".format(c) for c in sbuf))  
				idx += 1
				if idx == 21 and self.magicWord[1] == ch:  
					if disp:
						print("------rx data(iData)-----")
						print(":".join("{:02x}".format(c) for c in sbuf))  
					try: 
						frame = struct.unpack('<i', sbuf[7:11])
						target = int(sbuf[18])
						pcNum = struct.unpack('<H',sbuf[19:21])
						
						sh = subHeader(frame[0],target,pcNum[0])
						#if disp:
						#print(sh)
						lstate = 'idle'
						sbuf = b""
						idx = 0
						if disp:
							print("(jb)iData(1) -> idle state")
						if target == 0:
							return (True,sh,[])
						else:
							lstate = 'iTarget'
							#print('iData -> iTarget')
			
					except:
						lstate = 'idle'
						self.ops = []
						self.shrd = []
						
						if disp:
							print("(jb)---iData Exception---")
					
				elif idx > 23:
					lstate = 'idle'
					sbuf = b""
					print("*********data over(iData)*********")
					if disp:
						print("data over {:d} back to idle state".format(idx))
					frameNum = 0
					objMax = 0
					idx = 0
					ops = []
					return (False,[],[]) 
					
			
